/*			FINGER ACCESS				HTFinger.c
**			=============
** Authors:
**  ARB  Andrew Brooks
**
** History:
**	21 Apr 94   First version (ARB, from HTNews.c by TBL)
*/

#include "HTUtils.h"
#include "tcp.h"
#include "HTFinger.h"
#include "HTML.h"
#include "HTParse.h"
#include "HTFormat.h"

struct _HTStructured 
{
  CONST HTStructuredClass *	isa;
  /* ... */
};

/* #define TRACE 1 */

#define FINGER_PORT 79		/* See rfc742 */

/*	Module-wide variables
*/
PRIVATE int s;					/* Socket for FingerHost */
PRIVATE HTStructured * target;			/* The output sink */
PRIVATE HTStructuredClass targetClass;		/* Copy of fn addresses */

#define PUTC(c) (*targetClass.put_character)(target, c)
#define PUTS(s) (*targetClass.put_string)(target, s)
#define START(e) (*targetClass.start_element)(target, e, 0, 0)
#define END(e) (*targetClass.end_element)(target, e)


/*	Initialisation for this module
**	------------------------------
*/
PRIVATE BOOL initialized = NO;
PRIVATE BOOL initialize NOARGS
{
  s = -1;		/* Disconnected */
  return YES;
}



/*	Send Finger Command line to remote host & Check Response
**	--------------------------------------------------------
**
** On entry,
**	command	points to the command to be sent, including CRLF, or is null
**		pointer if no command to be sent.
** On exit,
**	Negative status indicates transmission error, socket closed.
**	Positive status is a Finger status.
*/


PRIVATE int response ARGS1(CONST char *,command)
{
  int status;
  int length = strlen(command);
  int result;    
  int ch;

  if (length==0) return(-1);
  if (TRACE) 
    fprintf(stderr, "Finger command to be sent: %s", command);
  status = NETWRITE(s, (char *)command, length);
  if (status<0)
    {
      if (TRACE) fprintf(stderr,
                         "HTFinger: Unable to send command. Disconnecting.\n");
      NETCLOSE(s);
      s = -1;
      return status;
    } /* if bad status */
  
  if (TRACE)
	fprintf(stderr,"Reading finger information\n");
  HTInitInput(s);
  START(HTML_PRE);
  while ((ch = HTGetCharacter()) != ((char)EOF)) PUTC(ch);
  END(HTML_PRE);
  if (TRACE)
	fprintf(stderr,"Closing net\n");
  NETCLOSE(s);
  s = -1;
  if (TRACE)
	fprintf(stderr,"exiting\n");
  return(0);
}


/*	Abort the connection					abort_socket
**	--------------------
*/
PRIVATE void abort_socket NOARGS
{
    if (TRACE) fprintf(stderr,
	    "HTFinger: EOF on read, closing socket %d\n", s);
    NETCLOSE(s);	/* End of file, close socket */
    PUTS("Network Error: connection lost");
    PUTC('\n');
    s = -1;		/* End of file on response */
    return;
}


/*		Load by name					HTLoadFinger
**		============
*/
PUBLIC int HTLoadFinger ARGS4(
	CONST char *,		arg,
	HTParentAnchor *,	anAnchor,
	HTFormat,		format_out,
	HTStream*,		stream)
{
  char *slash, *username, *sitename, *colon;    /* Fields extracted from URL */
  char str[1024];			/* Buffer */
  int port;				/* Port number from URL */
  int status;				/* tcp return */
  int retries;			/* A count of how hard we have tried */ 
  
  if (TRACE) 
    fprintf(stderr, "HTFinger: Looking for %s\n", arg);
  
  if (!initialized) 
    initialized = initialize();
  if (!initialized) 
    {
      HTAlert ("Could not set up finger connection.");
      return HT_NOT_LOADED;	/* FAIL */
    }
    
  {
    CONST char * p1=arg;
    
    /*	We will ask for the document, omitting the host name & anchor.
     **
     **	Syntax of address is
     **		xxx/yyy			User yyy at site xxx (yyy is optional).
     */        
    if (!strncasecomp (arg, "finger://", 9))
      p1 = arg + 9;  /* Skip "finger://" prefix */
    sitename = (char *)p1;

    if ((slash = strchr(p1, '/'))) {
      username = slash+1;
      *slash = '\0';
    } else {
      username = "";
    }
    
    if (strlen(sitename)==0)
      {
        HTAlert ("Could not load data (no sitename in finger URL)");
        return HT_NOT_LOADED;			/* Ignore if no name */
      }
    
    colon = strchr(sitename, ':');
    if (colon) 
      {
        port = atoi(colon+1);
        *colon = '\0';
        if (port != 79) 
          {
	    HTAlert ("Invalid port number - will only use port 79!");
            return HT_NOT_LOADED;		/* Ignore if wrong port */
          }
      }
  } /* scope of p1 */
  
  if (!*arg) 
    {
      HTAlert ("Could not load data.");
      return HT_NOT_LOADED;			/* Ignore if no name */
    }
  
  /*	Make a hypertext object with an anchor list. */       
  target = HTML_new(anAnchor, format_out, stream);
  targetClass = *target->isa;	/* Copy routine entry points */
  
  /*	Now, let's get a stream setup up from the FingerHost: */       
  /* CONNECTING to finger host */
  sprintf (str, "lose://%s/", sitename);
  if (TRACE)
    fprintf (stderr, "Finger: doing HTDoConnect on '%s'\n", str);
  status = HTDoConnect (str, "Finger", FINGER_PORT, &s);
  if (TRACE)
    fprintf (stderr, "Finger: Done DoConnect; status %d\n", status);
  if (status == HT_INTERRUPTED)
    {
      /* Interrupt cleanly. */
      if (TRACE)
        fprintf (stderr,
               "Finger: Interrupted on connect; recovering cleanly.\n");
      HTProgress ("Connection interrupted.");
      return HT_INTERRUPTED;
    }
  if (status < 0)
    {
      char message[256];
      NETCLOSE(s);
      s = -1;
      if (TRACE) 
        fprintf(stderr, "HTFinger: Unable to connect to finger host.\n");
      HTProgress ("Could not access finger host.");
      sprintf(message, "\nCould not access finger host %s.",
              sitename);
      PUTS(message);
      (*targetClass._free)(target);
      return HT_LOADED;
    } 
  else 
    {
      if (TRACE) fprintf(stderr, "HTFinger: Connected to finger host %s.\n",
                         sitename);
      HTInitInput(s);		/* set up buffering */
      if (*username) { 
        sprintf(str, "%s%c%c", HTUnEscape(username), CR, LF);
      } else {
        sprintf(str, "%c%c", CR, LF);
      }
      START(HTML_TITLE);
      PUTS("\nFinger search results");
      END(HTML_TITLE);
      if (response(str) != 0) 
        {
          NETCLOSE(s);
          s = -1;
          /* Couldn't get it. */
          START(HTML_TITLE);
          PUTS("Could Not Retrieve Information");
          END(HTML_TITLE);
          PUTS("Sorry, could not retrieve information.");
          (*targetClass._free)(target);
          return HT_LOADED;
        }
    }
      
  (*targetClass._free)(target);
  return HT_LOADED;
}

#ifdef GLOBALDEF_IS_MACRO
#define _HTFINGER_C_1_INIT { "finger", HTLoadFinger, NULL }
GLOBALDEF (HTProtocol, HTFinger, _HTFINGER_C_1_INIT);
#else
GLOBALDEF PUBLIC HTProtocol HTFinger = { "finger", HTLoadFinger, NULL };
#endif /* GLOBALDEF_IS_MACRO */
